//Initialization and setting configuration
#include <Wire.h>
#define SLAVE 0x50
#define NUM_SENSORS 4 // the code is written to be able to read 4 capacitive sensors
//set the offsets as variables
int OFFSET = 15;
double data_list[6] = { };


void setup() {
  //Calculate CAPDAC value for desired capacitive offset
  int capdac = round(OFFSET / 3.125); 
  Wire.begin(); //default for arduino, GPIO 4= SDA, GPIO 5= SCL
  Serial.begin(115200);

  //Configure inputs 1 to 4
  for (int i = 0; i < NUM_SENSORS; i++) {
      //Caculate register value from CAPDAC value
      int register_value = 0x1000 + (capdac << 5); 
      Wire.beginTransmission(SLAVE);
      Wire.write(0x08 + i); //address the measurement config register for the 4 channels: 0x08,0x09,0x0A,0x0B
      Wire.write(highByte(register_value) + 32 * i);
      Wire.write(lowByte(register_value));
      Wire.endTransmission();
      //Settings note from datasheet:
      //CHA bits [15:13] --> positive value
      //CHB bits [12:10] --> negative value
      //but if CHB=100 --> CAPDAC enabled
      //bits [9:5] --> CAPDAC value
  }

  //Configure measurement settings, with FDC Configuration Register 0x0C
  Wire.beginTransmission(SLAVE);
  Wire.write(0x0C); //configuration register address
  Wire.write(0x0D); //sampling rate of 400 S/s --> ( 100S/s = 0x05; 200S/s = 0x09; 400S/s = 0x0D)
  Wire.write(0xF0); //initiate measurements 1-2-3-4 to be able to read all four channels
  Wire.endTransmission();
}

// --------------------------LOOP, RUN MULTIPLE TIMES -------------------
void loop() {
  Serial.println("");
  //check if ready, start the measurement
  if (isReady(0)) {
    data_list[0] = readC(0);
  }

  if (isReady(1)) {
    data_list[1] = readC(1);
  }

  if (isReady(2)) {
    data_list[2] = readC(2);
  }

  if (isReady(3)) {
    data_list[3] = readC(3);
  }

  for (int i = 0; i < NUM_SENSORS; i++) {
    //now I print the found measurements for all the channels
    Serial.print("CH" + String(i) + ":" + String(data_list[i], 5) + " " );
  }
}

//Input: Number of input pin (starts at 0)
//Output: True if measurement at position ch_n is completed, false otherwise
bool isReady(int ch_n) { //channel in input --> returns TRUE if a measurement is completed
  int fdc[2];
  int i = 0;
  Wire.beginTransmission(SLAVE);
  Wire.write(0x0C); //address FDC config measurement to get the value of interest
  Wire.endTransmission();
  Wire.requestFrom(SLAVE, 2);

  while (Wire.available()) { //Returns the number of bytes available for retrieval with read().
    fdc[i] = Wire.read();
    i++;
  }
  return bitRead(fdc[1], 3 - ch_n); 
}

float voltage1 = 0; //variable used to convert the value from binary to decimal
float cap1 = 0;

//Input: Number of input pin (starts at 0)
//Output: capacitance at input ch_n to serial
double readC(int ch_n) {
  int c;
  int c_data[2]; // I want to save first the MSB and then the LSB
  Wire.beginTransmission(SLAVE);
  Wire.write(0x00 + ch_n * 2); // at each channel I address the MSB for that measurement: 0x00-CH1,0x02-CH2,0x04-CH3,0x06-CH4
  Wire.endTransmission();
  Wire.requestFrom(SLAVE, 2);

  for (int i = 0; i < 2; i++) {
    c_data[i] = Wire.read(); //read both requested bytes
  }
  //Convert byte array to int and print capacitance to serial
  c = (c_data[0] << 8) + c_data[1]; // 16 bit value --> binary output
  
  //conversion to decimal
  voltage1 = c * (30000.0 / 65535.0); //remapping
  cap1 = voltage1 / 1000 + 5 * 3.125; //put in voltage value
  return cap1;

}

